#ifndef _LIBOJMIXER_OJMIXER_H_
#define _LIBOJMIXER_OJMIXER_H_

#include <glib.h>
#include <glib-object.h>

typedef struct _OjMixer      OjMixer;
typedef struct _OjMixerClass OjMixerClass;

#include <libojmixer/ojpreset.h>

struct _OjMixer
{
	GObject parent_instance;

	int fd;

	int devmask;
	int recmask;
	int stereodevs;

	gboolean fade_on_switch;
	gboolean is_fading;

	GList *presets;
};

struct _OjMixerClass
{
	GObjectClass parent_class;

	void (*volume_changed)(OjMixer *mixer, guint device, guint volume);
	void (*recsrc_changed)(OjMixer *mixer, guint device);
	void (*preset_added)(OjMixer *mixer, OjPreset *preset);
	void (*preset_removed)(OjMixer *mixer, OjPreset *preset);
	void (*preset_switched)(OjMixer *mixer, OjPreset *preset);
};

#define TYPE_OJ_MIXER (oj_mixer_get_type())
#define OJ_MIXER(object) (G_TYPE_CHECK_INSTANCE_CAST ((object), TYPE_OJ_MIXER, OjMixer))
#define OJ_MIXER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_OJ_MIXER, OjMixerClass))
#define IS_OJ_MIXER(object) (G_TYPE_CHECK_INSTANCE_TYPE ((object), TYPE_OJ_MIXER))
#define IS_OJ_MIXER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_OJ_MIXER))
#define OJ_MIXER_GET_CLASS(object) (G_TYPE_INSTANCE_GET_CLASS ((object), TYPE_OJ_MIXER, OjMixerClass))

GType oj_mixer_get_type(void);

OjMixer *oj_mixer_new(void);

gboolean oj_mixer_open(OjMixer *mixer, const char *dev);

int oj_mixer_get_status(OjMixer *mixer);
int oj_mixer_get_volume(OjMixer *mixer, int device, int *left,
					   int *right, int *volume);
int oj_mixer_get_rec_source(OjMixer *mixer);
int oj_mixer_get_device_count(OjMixer *mixer);
gboolean oj_mixer_is_device_supported(OjMixer *mixer, int device);
gboolean oj_mixer_can_device_record(OjMixer *mixer, int device);
gboolean oj_mixer_device_has_stereo(OjMixer *mixer, int device);
const char *oj_mixer_get_device_name(OjMixer *mixer, int device);
const char *oj_mixer_get_device_label(OjMixer *mixer, int device);

int oj_mixer_set_volume(OjMixer *mixer, int device, int left_right);
int oj_mixer_set_rec_source(OjMixer *mixer, int recsrc);

void oj_mixer_set_fade_on_switch(OjMixer *mixer, gboolean value);
gboolean oj_mixer_get_fade_on_switch(const OjMixer *mixer);

void oj_mixer_switch_preset(OjMixer *mixer, OjPreset *preset, gboolean fade);

void oj_mixer_add_preset(OjMixer *mixer, OjPreset *preset);
void oj_mixer_remove_preset(OjMixer *mixer, OjPreset *preset);

OjPreset *oj_mixer_find_preset(OjMixer *mixer, const char *name);
gboolean oj_mixer_is_fading(OjMixer *mixer);

#endif /* _LIBOJMIXER_OJMIXER_H_ */
