#include <libojmixer/ojpreset.h>
#include "oj-marshal.h"

static void oj_preset_class_init(GObjectClass *klass, gpointer data);
static void oj_preset_instance_init(GObject *object, gpointer data);
static void oj_preset_instance_finalize(GObject *object);

enum
{
	VOLUME_CHANGED,
	RECSRC_CHANGED,
	LAST_SIGNAL
};

static guint signals[LAST_SIGNAL] = { 0 };

GType
oj_preset_get_type(void)
{
	static GType oj_preset_type = 0;

	if (!oj_preset_type)
	{
		static const GTypeInfo oj_preset_info =
		{
			sizeof(OjPresetClass),
			NULL,
			NULL,
			(GClassInitFunc)oj_preset_class_init,
			NULL,
			NULL,
			sizeof(OjPreset),
			0,
			(GInstanceInitFunc)oj_preset_instance_init
		};

		oj_preset_type = g_type_register_static(G_TYPE_OBJECT, "OjPreset",
											   &oj_preset_info, 0);
	}

	return oj_preset_type;
}

static void
oj_preset_class_init(GObjectClass *klass, gpointer data)
{
	GObjectClass *object_class;

	object_class = G_OBJECT_CLASS(klass);
	object_class->finalize = oj_preset_instance_finalize;

	signals[VOLUME_CHANGED] =
		g_signal_new("volume_changed",
					 G_TYPE_FROM_CLASS(object_class),
					 G_SIGNAL_RUN_LAST,
					 G_STRUCT_OFFSET(OjPresetClass, volume_changed),
					 NULL, NULL,
					 oj_marshal_VOID__UINT_UINT,
					 G_TYPE_NONE, 2,
					 G_TYPE_UINT, G_TYPE_UINT);

	signals[RECSRC_CHANGED] =
		g_signal_new("recsrc_changed",
					 G_TYPE_FROM_CLASS(object_class),
					 G_SIGNAL_RUN_LAST,
					 G_STRUCT_OFFSET(OjPresetClass, recsrc_changed),
					 NULL, NULL,
					 g_cclosure_marshal_VOID__UINT,
					 G_TYPE_NONE, 1,
					 G_TYPE_UINT);
}

static void
oj_preset_instance_init(GObject *object, gpointer data)
{
}

static void
oj_preset_instance_finalize(GObject *object)
{
	GObjectClass *parent_class;
	OjPreset *preset;

	preset = OJ_PRESET(object);

	g_free(preset->name);

	parent_class = g_type_class_peek_parent(G_OBJECT_GET_CLASS(object));
	parent_class->finalize(object);
}

OjPreset *
oj_preset_new(void)
{
	return g_object_new(TYPE_OJ_PRESET, NULL);
}

void
oj_preset_set_name(OjPreset *preset, const char *name)
{
	g_return_if_fail(preset != NULL);

	if (preset->name != NULL)
		g_free(preset->name);

	preset->name = (name == NULL ? NULL : g_strdup(name));
}

void
oj_preset_set_volume(OjPreset *preset, int device, int volume)
{
	g_return_if_fail(preset != NULL);

	preset->devices[device] = volume;

	g_signal_emit(preset, signals[VOLUME_CHANGED], 0, device, volume);
}

void
oj_preset_set_rec_device(OjPreset *preset, int device)
{
	g_return_if_fail(preset != NULL);

	preset->rec_device = device;

	g_signal_emit(preset, signals[RECSRC_CHANGED], 0, device);
}

void
oj_preset_load_volume(OjPreset *preset, OjMixer *mixer)
{
	int i;

	g_return_if_fail(preset != NULL);
	g_return_if_fail(mixer  != NULL);

	for (i = 0; i < SOUND_MIXER_NRDEVICES; i++)
	{
		if (oj_mixer_is_device_supported(mixer, i))
		{
			int volume;

			if (oj_mixer_get_volume(mixer, i, NULL, NULL, &volume) != 0)
				continue; /* FIXME - Display an error? */

			oj_preset_set_volume(preset, i, volume);
		}
	}
}

void
oj_preset_load_rec_device(OjPreset *preset, OjMixer *mixer)
{
	int source = 0;

	g_return_if_fail(preset != NULL);
	g_return_if_fail(mixer  != NULL);

	source = oj_mixer_get_rec_source(mixer);

	if (source != -1)
		oj_preset_set_rec_device(preset, source);
}

const char *
oj_preset_get_name(const OjPreset *preset)
{
	g_return_val_if_fail(preset != NULL, NULL);

	return preset->name;
}

int
oj_preset_get_volume(const OjPreset *preset, int device)
{
	g_return_val_if_fail(preset != NULL, -1);

	return preset->devices[device];
}

int
oj_preset_get_rec_device(const OjPreset *preset)
{
	g_return_val_if_fail(preset != NULL, -1);

	return preset->rec_device;
}

