/**
 * @file args.h Parses arguments
 * 
 * $Id: args.h,v 1.1 2003/04/08 20:26:38 chipx86 Exp $
 *
 * @Copyright (C) 2001-2003 The GNUpdate Project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA  02111-1307  USA
 */
#ifndef _OJ_ARGS_H_
#define _OJ_ARGS_H_

typedef enum
{
	CMD_UNSPECIFIED,
	CMD_LOAD,
	CMD_SAVE

} Command;

/**
 * The type of value in the Argument structure.
 *
 * @see Argument
 */
typedef enum
{
	ARG_INT,      /**< An integer. */
	ARG_STRING,   /**< A string.   */
	ARG_COMMAND,  /**< A Command.  */
	ARG_FUNCTION, /**< A function. */
	ARG_SETTING   /**< A setting.  */

} ArgType;

typedef struct _Argument Argument;  /**< Argument structure */

/**
 * Represents a supported argument.
 */
struct _Argument
{
	char  shortOpt;   /**< The short option (-f)       */
	char *longOpt;    /**< The long option (--foobar)  */
	char *commandOpt; /**< The command option (foobar) */

	void *var;        /**< The variable to set upon encountering the flag. */

	union
	{
		int integer;     /**< The integer value. */
		char *string;    /**< The string value.  */
		Command command; /**< The command value. */

	} value;

	/** The function to call, if ARG_FUNCTION is passed to type. */
	void (*fnc)(Argument *arg);
	
	ArgType type;     /**< The type of value. */
};

/**
 * Parses arguments in an array of strings.
 *
 * @param argc             The number of items in the incoming array.
 * @param argv             The incoming array of strings.
 * @param argTypeCount     The number of passed Argument structures.
 * @param argTypes         The passed array of Argument structures.
 * @param newArgc          The new count of the outgoing array.
 * @param newArgv          The new outgoing array of extra/unknown parameters.
 * @param keepUnknownFlags If 1, put all unknown flags in the outgoing array.
 *                         If 0, generate an error when one is encountered.
 */
void parseArgs(int argc, char **argv, int argTypeCount, Argument *argTypes,
			   int *newArgc, char ***newArgv, char keepUnknownFlags);

#endif /* _OJ_ARGS_H_ */

