#include <stdio.h>
#include <stdlib.h>

#include <libojmixer/libojmixer.h>
#include "ojmixerwin.h"
#include "stock.h"
#include "args.h"
#include "config.h"
#include "settings.h"

static OjMixer *mixer = NULL;

static char **newArgv = NULL;
static int newArgc = 0;

static int _fade = 0;

static void
__help(Argument *arg)
{
	printf("OJ Mixer v%s\n", VERSION);
	printf(_("Copyright (C) 2003 Christian Hammond.\n"));

	printf("\n");

	printf(_("Usage: ojmixer [OPTION...] [COMMAND [OPTION]]\n"));

	printf("\n");

	printf(_("General OJ Mixer information:\n"));
	printf(_("  -h, --help,             - Displays this help screen.\n"
			 "        help\n"));
	
	printf("\n");

	printf(_("Options:\n"));
	printf(_("  -f, --fade              - Fades between a preset when using"
			 "--load\n"));

	printf("\n");

	printf(_("Commands:\n"));
	printf(_("  -l, --load,             - Loads the specified preset.\n"
			 "        load\n"));
	printf(_("  -s, --save,             - Saves the current mixer settings to the\n"
			 "        save                specified preset.\n"));
	
	if (newArgv != NULL)
		free(newArgv);

	exit(1);
}

static void
__setCommand(Argument *arg)
{
	setCommand(arg->value.command);
}

static void
__setFade(Argument *arg)
{
	_fade = arg->value.integer;
}

static Argument args[] =
{
	{ 'h', "help", "help", NULL, {0},        __help,       ARG_FUNCTION },
	{ 'l', "load", "load", NULL, {CMD_LOAD}, __setCommand, ARG_FUNCTION },
	{ 's', "save", "save", NULL, {CMD_SAVE}, __setCommand, ARG_FUNCTION },
	{ 'f', "fade", NULL,   NULL, {1},        __setFade,    ARG_FUNCTION }
};

static void
load_preset(int argc, char **argv)
{
	OjPreset *preset;
	int device;
	GMainContext *ctx;

	if (argc < 1)
	{
		fprintf(stderr, _("You must specify a valid preset to switch to.\n"));
		return;
	}
	else if (argc > 1)
	{
		fprintf(stderr, _("You may specify only one preset to switch to.\n"));
		return;
	}

	preset = oj_mixer_find_preset(mixer, argv[0]);

	if (preset == NULL)
	{
		fprintf(stderr, _("The specified preset was not found.\n"));
		return;
	}

	device = oj_preset_get_rec_device(preset);

	oj_mixer_switch_preset(mixer, preset, _fade);
	oj_mixer_set_rec_source(mixer, device);

	if (_fade)
	{
		ctx = g_main_context_default();

		while (oj_mixer_is_fading(mixer))
			g_main_context_iteration(ctx, FALSE);
	}
}

static void
save_preset(int argc, char **argv)
{
	OjPreset *preset;

	if (argc < 1)
	{
		fprintf(stderr, _("You must specify a name for the new preset.\n"));
		return;
	}
	else if (argc > 1)
	{
		fprintf(stderr, _("You may specify only one preset to save to.\n"));
		return;
	}

	preset = oj_mixer_find_preset(mixer, argv[0]);

	if (preset == NULL)
	{
		preset = oj_preset_new();

		oj_mixer_add_preset(mixer, preset);
		oj_preset_set_name(preset, argv[0]);
	}

	oj_preset_load_volume(preset, mixer);
	oj_preset_load_rec_device(preset, mixer);

	oj_config_save(mixer);
}

int
main(int argc, char **argv)
{
	OjMixerWindow *win;

	g_type_init();

	mixer = oj_mixer_new();
	oj_mixer_open(mixer, "/dev/mixer");
	oj_config_load(mixer);

	if (argc > 1)
	{
		parseArgs(argc - 1, argv + 1, sizeof(args) / sizeof(*args), args,
				  &newArgc, &newArgv, 1);

		switch (getCommand())
		{
			case CMD_LOAD:
				load_preset(newArgc, newArgv);
				break;

			case CMD_SAVE:
				save_preset(newArgc, newArgv);
				break;

			default:
				fprintf(stderr, _("Unrecognized option\n"));
				break;
		}
	}
	else
	{
		gtk_init(&argc, &argv);

		setup_stock();

		win = oj_mixer_window_new(mixer);

		gtk_widget_show(win->window);

		gtk_main();
	}

	return 0;
}

